@@ -23,8 +23,9 @@
 #include "input.h"
 #include "color.h"
 #include "util.h"
+#include "win.h"
 
-static char rcsid [] = "$Header: /usr/users/miguel/c/CVS/nc/dialog.c,v 1.2 1994/05/14 21:34:41 miguel Exp $";
+static char rcsid [] = "$Header: /usr/users/miguel/c/CVS/nc/dialog.c,v 1.3 1994/06/27 22:52:51 miguel Exp $";
 static char buffer [4096];
 
 static Dialog *current_dialog = 0;
@@ -32,9 +33,8 @@ WINDOW *top_window;
 
 void (*refresh_fn)();
 
-create_dialog (int cols, int rows, char *header, char *text, int error)
+void create_dialog (int cols, int rows, char *header, char *text, int error)
 {
-    int key;
     int base = LINES/3;
     Dialog *newd;
     
@@ -129,6 +129,7 @@ int message (int error, char *header, char *text,  ...)
 	done_dialog ();
 	return v;
     }
+    return 0; 
 }
 
 int error (int status, int errnum, char *text,  ...)
@@ -151,13 +152,13 @@ int error (int status, int errnum, char *text,  ...)
     v = run_dialog ();
     done_dialog ();
     va_end (args);
+    return 0;
 }
 
 char *input_dialog (char *header, char *text, char *def_text)
 {
     int    len, lines;
     char   *return_value;
-    static char inbuf [58];
     Input  in;
     int    c;
 
@@ -166,13 +167,14 @@ char *input_dialog (char *header, char *text, char *def_text)
     len = max (strlen (header), msglen (buffer, &lines));
     len = max (len, 60);
     create_dialog (len, lines+1, header, buffer, 0);
-    create_input (&in, 1, lines, sizeof (inbuf)-1, inbuf, current_dialog->t,
-		  INPUT_COLOR, sizeof (inbuf)-1, def_text);
+    create_input (&in, 1, lines, current_dialog->t, INPUT_COLOR,
+		  58, def_text);
 
-    return_value = inbuf;
+    return_value = in.buffer;
     while ((c = mi_getch ()) != '\n'){
 	if (c == '\e'){
 	    return_value = 0;
+	    free (in.buffer);
 	    break;
 	}
 	handle_char (&in, c);
