@@ -12,7 +12,9 @@ extern WINDOW *top_window;
 char *input_dialog (char *header, char *text, char *def_text);
 int message (int error, char *header, char *text, ...);
 int error (int status, int errnum, char *text,  ...);
-int create_dialog (int cols, int rows, char *header, char *text, int error);
+void create_dialog (int cols, int rows, char *header, char *text, int error);
+int query_dialog (char *header, char *text, int flags, int count, ...);
+void done_dialog ();
 
 #define D_NORMAL 0
 #define D_ERROR  1
