@@ -24,11 +24,15 @@
 #include "win.h"
 
 #define MAXLINKNAME 80
+#define HISTORY_SIZE 20
 
-static char rcsid [] = "$Header: /usr/users/miguel/c/CVS/nc/help.c,v 1.3 1994/05/14 21:34:46 miguel Exp $";
-char *data;
+static char rcsid [] = "$Header: /usr/users/miguel/c/CVS/nc/help.c,v 1.5 1994/06/27 22:53:05 miguel Exp $";
+static char *data;
 static int help_lines = 18;
 WINDOW *whelp;
+static char *history [HISTORY_SIZE];
+static int  history_ptr;
+static char *main;
 
 char *search_string (char *start, char *text)
 {
@@ -50,8 +54,11 @@ static char *search_string_node (char *start, char *text)
 {
     char *d = text;
     char *e = start;
+
+    if (!start)
+	return main;
     
-    for (; *e && *e != '%'; e++){
+    for (; *e && *e != '\x4'; e++){
 	if (*d == *e)
 	    d++;
 	else
@@ -67,7 +74,7 @@ static char *move_forward (char *current, int lines)
     char *p;
     int  line;
 
-    for (line = 0, p = current; *p && *p != '%'; p++){
+    for (line = 0, p = current; *p && *p != '\x4'; p++){
 	if (*p == '\n')
 	    line++;
 	if (line == lines)
@@ -81,7 +88,7 @@ static char *move_backward (char *current, int lines)
     char *p;
     int line;
 
-    for (line = 0, p = current; *p && *p != '%' && p > data; p--){
+    for (line = 0, p = current; *p && *p != '\x4' && p > data; p--){
 	if (*p == '\n')
 	    line++;
 	if (line == lines)
@@ -90,19 +97,20 @@ static char *move_backward (char *current, int lines)
     return current;
 }
 
-static char *follow_link (char *selected_item)
+static char *follow_link (char *start, char *selected_item)
 {
     char link_name [MAXLINKNAME];
     char *p;
     int  i = 0;
 
     if (!selected_item)
-	return;
-    for (p = selected_item; *p && *p != '%' && *p != '\002'; p++)
+	return start;
+    
+    for (p = selected_item; *p && *p != '\x4' && *p != '\002'; p++)
 	;
-    if (*p = '\002'){
+    if (*p == '\002'){
 	link_name [0] = '[';
-	for (i = 1; *p != '\003' && *p && *p != '%' && i < MAXLINKNAME-3; )
+	for (i = 1; *p != '\003' && *p && *p != '\x4' && i < MAXLINKNAME-3; )
 	    link_name [i++] = *++p;
 	link_name [i-1] = ']';
 	link_name [i] = 0;
@@ -110,7 +118,7 @@ static char *follow_link (char *selected_item)
 	if (p)
 	    return p;
     }
-    return "Help file format error\n%";	/*  */
+    return "Help file format error\n\x4";	/*  */
 }
 
 static char *select_next_link (char *start, char *current_link)
@@ -129,7 +137,7 @@ static char *select_next_link (char *start, char *current_link)
     return 0;
 }
 
-static show (char *paint_start, char *selected_start)
+static void show (char *paint_start, char *selected_start)
 {
     char *p;
     int  col, line;
@@ -138,7 +146,7 @@ static show (char *paint_start, char *selected_start)
     line = col = 0;
     werase (whelp);
     wclr (whelp);
-    for (p = paint_start; *p != '%' && line < help_lines; p++){
+    for (p = paint_start; *p != '\x4' && line < help_lines; p++){
 	if (*p == 1){
 	    if (p == selected_start)
 		wattrset (whelp, MARKED_COLOR | A_BOLD);
@@ -166,7 +174,7 @@ static show (char *paint_start, char *selected_start)
 void interactive_display (char *filename)
 {
     int quit = 0;
-    char *current, *start, *main, *selected_item;
+    char *current, *start, *selected_item;
     
     if ((data = load_file (filename)) == 0)
 	return;
@@ -178,6 +186,8 @@ void interactive_display (char *filename)
     }
     selected_item = search_string_node (main, "\001") - 1;
     current = start = main;
+    for (history_ptr = 0; history_ptr < HISTORY_SIZE; history_ptr++)
+	history [history_ptr] = current;
     do {
 	show (current, selected_item);
 	switch (mi_getch ()){
@@ -201,12 +211,18 @@ void interactive_display (char *filename)
 	case KEY_UP:
 	    current = move_backward (current, 2);
 	    break;
+
+	case 'l':
+	    current = start = history [--history_ptr % HISTORY_SIZE];
+	    break;
 	    
 	case '\n':
-	    if (!selected_item)
-		current = start = main;
-	    else
-		current = start = follow_link (selected_item);
+	    if (!selected_item){
+		current = start = history [--history_ptr % HISTORY_SIZE];
+	    } else {
+		history [history_ptr++ % HISTORY_SIZE] = current;
+		current = start = follow_link (current, selected_item);
+	    }
 	    selected_item = search_string_node (current, "\001");
 	    if (selected_item)
 		selected_item--;
