@@ -15,6 +15,7 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #include <ncurses.h>
+#include <ctype.h>
 #include "global.h"
 #include "main.h"
 
@@ -71,7 +72,8 @@ int mi_getch ()
 	return KEY_END;
 	
     default:
-	if ((d >= 'a' && d <= 'z') || (d >= 'A' && d <= 'Z' ) || d == '\n')
+	if ((d >= 'a' && d <= 'z') || (d >= 'A' && d <= 'Z' )
+	    || d == '\n' || d == '\t')
 	    return 0x80 | d;
 	else
 	    return d;
