@@ -25,6 +25,7 @@
 #include <errno.h>
 #include <pwd.h>
 #include <unistd.h>
+#include <fcntl.h>		/* For O_RDWR */
 #include <signal.h>
 #include "dir.h"
 #include "color.h"
@@ -39,7 +40,7 @@
 #include "win.h"
 #include "user.h"
 
-static char rcsid [] = "$Header: /usr/users/miguel/c/CVS/nc/main.c,v 1.3 1994/05/14 21:34:51 miguel Exp $";
+static char rcsid [] = "$Header: /usr/users/miguel/c/CVS/nc/main.c,v 1.5 1994/06/27 22:53:09 miguel Exp $";
 
 /* The structures for the panels */
 Panel left_panel, right_panel;
@@ -48,7 +49,6 @@ Panel left_panel, right_panel;
 Panel *current_panel, *other_panel;
 
 Input cmdline;
-char  input_buffer [255];
 
 /* Usefull macros to avoid too much typing */
 #define cpanel current_panel
@@ -75,6 +75,12 @@ int quote = 0;
 /* If true use the internal viewer */
 int use_internal_view = 1;
 
+/* Have we shown the fast-reload warning in the past? */
+int fast_reload_w = 0;
+
+/* If true all typed chars go to the incremental search routine */
+int searching = 0;
+
 char *prompt = "$ ";
 
 /* For slow terminals */
@@ -97,10 +103,14 @@ char *home_dir;
 /* The value of the other directory, only used when loading the setup */
 char *other_dir = 0;
 
+/* If true, then print on stdout the last directory we were at */
+int print_last_wd = 0;
+
 /* Ugly hack in order to distinguish between left and right panel in menubar */
 int is_right;
 #define MENU_PANEL  (is_right ? &right_panel : &left_panel)
 #define OTHER_PANEL (is_right ? &left_panel : &right_panel)
+
 /* Quick search status */
 char search_buffer [256] = { 0 };
 char cmd_buf [512];
@@ -116,11 +126,14 @@ inline void unselect_item (Panel *panel)
     panel_refresh (cpanel);
 }
 
-/* This function sets the selection variable */
+#define SELECT_ITEM(panel) \
+    selection = &current_panel->dir [current_panel->selected]; \
+    set_attr (cpanel, 1, selection->f.marked);
+
+/* This function sets the selection variable and redisplays the data */
 inline void select_item (Panel *panel)
 {
-    selection = &current_panel->dir [current_panel->selected];
-    set_attr (cpanel, 1, selection->f.marked);
+    SELECT_ITEM (panel);
     repaint_file (cpanel, cpanel->selected);
     panel_refresh (cpanel);
     display_mini_info (cpanel);
@@ -184,7 +197,8 @@ void init_labels ()
 void try_to_select (Panel *panel, char *name)
 {
     Xtry_to_select (panel, name);
-    select_item (panel);
+    SELECT_ITEM (panel);
+    display_mini_info (panel);
 }
 
 /* This routine reloads the directory in both panels. It tries to
@@ -202,7 +216,7 @@ void update_panels (int force_update, char *current_file, char *other_file)
     }
     
     /* If current_file == -1 (an invalid pointer, then preserve selection */
-    if (current_file == (char *) -1){
+    if (current_file == UP_KEEPSEL){
 	free_pointers = 1;
 	current_file = strdup (cpanel->dir [cpanel->selected].fname);
 	other_file   = strdup (opanel->dir [opanel->selected].fname);
@@ -233,12 +247,14 @@ void pre_exec ()
     }
     keypad (stdscr, FALSE);
     reset_shell_mode ();
+    endwin ();
 }
 
 /* Restores the terminal after an exec, see execute for extra post-exec code */
 void post_exec ()
 {
     reset_prog_mode ();
+    flushinp ();
     cbreak ();
     keypad (stdscr, TRUE);
 }
@@ -247,26 +263,32 @@ void post_exec ()
 /* when no modifications have took place */
 void save_cwds_stat ()
 {
-    stat (cpanel->cwd, &(cpanel->dir_stat));
-    stat (opanel->cwd, &(opanel->dir_stat));
+    if (fast_reload){
+	stat (cpanel->cwd, &(cpanel->dir_stat));
+	stat (opanel->cwd, &(opanel->dir_stat));
+    }
 }
 
 void execute (char *command)
 {
-    int  i;
+#ifdef OLD_SYSTEM_CODE
     char buf [4096];
-#ifdef OLD_CODE
-    char *current_file = strdup (selection->fname);
-    char *other_file   = strdup (selection->fname);
 #endif
     struct passwd *pw_data;
 
     pw_data = getpwuid (geteuid ());
-    sprintf (buf, "%s -c \"%s\"", pw_data->pw_shell, command);
 
+#ifdef OLD_SYSTEM_CODE
+    sprintf (buf, "%s -c \"%s\"", pw_data->pw_shell, command);
+#endif
+    
     save_cwds_stat ();
     pre_exec ();
+#ifdef OLD_SYSTEM_CODE
     system (buf);
+#else
+    my_system (pw_data->pw_shell, command);
+#endif
     post_exec ();
 
     if (pause_after_run){
@@ -274,15 +296,8 @@ void execute (char *command)
 	fflush (stdout);
 	getch ();
     }
-#ifdef OLD_CODE
-    update_panels (current_file, other_file);
-#endif
-    update_panels (UP_OPTIMIZE, (char *) -1, 0);
-    repaint_screen (RP_CLEAR);
-#ifdef OLD_CODE
-    free (current_file);
-    free (other_file);
-#endif
+    update_panels (UP_OPTIMIZE, UP_KEEPSEL, 0);
+    refresh_screen ();
 }
 
 /* Changes the currently selected panel to the other panel */
@@ -424,12 +439,17 @@ void mark_file ()
     if (selection->f.marked){
 	cpanel->marked++;
 	cpanel->total += selection->buf.st_size;
+	set_colors (cpanel);
+	if (cpanel->marked == 1)
+	    mvwprintw (cpanel->win_file, cpanel->lines+3, 1,
+		       "%-*s", cpanel->cols, "", COLS/2-2);
     } else {
 	cpanel->marked--;
 	cpanel->total -= selection->buf.st_size;
     }
     if (mark_moves_down)
 	move_down ();
+    display_mini_info (cpanel);
     panel_refresh (cpanel);
 }
 
@@ -475,16 +495,21 @@ void repaint_screen_cmd ()
     repaint_screen (RP_CLEAR);
 }
 
+void only_refresh_screen ()
+{
+    panel_refresh (current_panel);
+    panel_refresh (other_panel);
+    wrefresh (cmdline_win);
+    wrefresh (fkeys);
+}
+
 void refresh_screen ()
 {
     touchwin (current_panel->win_file);
     touchwin (other_panel->win_file);
     touchwin (cmdline_win);
     touchwin (fkeys);
-    panel_refresh (current_panel);
-    panel_refresh (other_panel);
-    wrefresh (cmdline_win);
-    wrefresh (fkeys);
+    only_refresh_screen ();
 }
 
 void help_cmd ()
@@ -505,9 +530,9 @@ int do_cd (char *new_dir)
     clean_dir (&cpanel->dir, cpanel->count);
     cpanel->count = do_load_dir (&cpanel->dir, cpanel->sort_type);
     cpanel->top_file = 0;
-
     cpanel->selected = 0;
     cpanel->marked = 0;
+    cpanel->total = 0;
     getwd (cpanel->cwd);
     return 1;
 }
@@ -560,32 +585,42 @@ void enter ()
     regex_command (selection->fname);
 }
 
+void do_search (int c_code)
+{
+    int l, i;
+    
+    l = strlen (search_buffer);
+    search_buffer [l] = c_code & 0x7f;
+    search_buffer [l+1] = 0;
+    l++;
+
+    for (i = 0; i < cpanel->count; i++){
+	if (strncmp (cpanel->dir [i].fname, search_buffer, l) == 0){
+	    cpanel->selected = i;
+	    cpanel->top_file = cpanel->selected - cpanel->lines/2;
+	    if (cpanel->top_file < 0)
+		cpanel->top_file = 0;
+
+	    select_item (cpanel);
+	    paint_panel (cpanel);
+	    break;
+	}
+    }
+}
+
+void start_search ()
+{
+    searching = 1;
+}
+
 /* If the key pressed is not bound to any command, this function is called */
 void default_key (int c_code)
 {
-    int i;
-    int l;
-    
-    if (c_code & 0x80 || search_buffer [0]){
-	l = strlen (search_buffer);
-	search_buffer [l] = c_code & 0x7f;
-	search_buffer [l+1] = 0;
-	l++;
-	
-	for (i = 0; i < cpanel->count; i++){
-	    if (strncmp (cpanel->dir [i].fname, search_buffer, l) == 0){
-		cpanel->selected = i;
-		cpanel->top_file = cpanel->selected - cpanel->lines/2;
-		if (cpanel->top_file < 0)
-		    cpanel->top_file = 0;
-		
-		select_item (cpanel);
-		paint_panel (cpanel);
-		break;
-	    }
-	}
-    } else 
+    if (searching)
+	do_search (c_code);
+    else {
 	handle_char (&cmdline, c_code);
+    }
 }
 
 void view_cmd ()
@@ -607,7 +642,7 @@ void view_cmd ()
     }
 }
 
-void do_edit ()
+void do_edit (const char *what)
 {
     static char *editor = 0;
 
@@ -616,7 +651,7 @@ void do_edit ()
 	if (!editor)
 	    editor = "vi";
     }
-    sprintf (cmd_buf, "%s %s", editor, selection->fname);
+    sprintf (cmd_buf, "%s %s", editor, what);
     execute (cmd_buf);
 }
 
@@ -636,6 +671,9 @@ char *get_file (Panel *panel)
 		return panel->dir [i].fname;
     } else
 	return panel->dir [panel->selected].fname;
+    fprintf (stderr, "Internal error\n");
+    mi_getch ();
+    return "";
 }
 
 /* Clears all files in the panel, used only when one file was marked */
@@ -649,6 +687,7 @@ void unmark_file (Panel *panel)
     for (i = 0; i < panel->count; i++)
 	panel->dir [i].f.marked = 0;
     panel->marked = 0;
+    panel->total = 0;
 }
 
 void copy_cmd ()
@@ -673,6 +712,7 @@ void copy_cmd ()
     if (S_ISREG (s.st_mode) || stat_r != 0){
 	if (cpanel->marked > 1){
 	    message (1, " Error ", "Can't copy multiple files to one file");
+	    free (dest);
 	    return;
 	}
 	copy_file_file (get_file (cpanel), dest);
@@ -680,6 +720,7 @@ void copy_cmd ()
     } else {
 	if (!S_ISDIR (s.st_mode)){
 	    message (1, " Copy ", "Unknown copy destination");
+	    free (dest);
 	    return;
 	}
 	/* Destination is a directory */
@@ -701,7 +742,8 @@ void copy_cmd ()
 	    nodelay (stdscr, FALSE);
 	}
     }
-    update_panels (UP_OPTIMIZE, (char *) -1, 0);
+    free (dest);
+    update_panels (UP_OPTIMIZE, UP_KEEPSEL, 0);
     repaint_screen (RP_NOCLEAR);
 }
 
@@ -710,13 +752,16 @@ void ren_cmd ()
     struct stat s;
     int i, c, stat_r;
     
-    char *new_name = input_dialog (" Rename/Move ", "Enter rename mask:", "");
+    char *new_name = input_dialog (" Rename/Move ", "Enter rename mask:",
+				   opanel->cwd);
     if (!new_name)
 	return;
     if (strchr (new_name, '*') || strchr (new_name, '?')){
 	/* Mask rename */
 	message (0, " Missing feature ",
 		 " Mask renaming not yet implemented, sorry ");
+	free (new_name);
+	return;
     }
     save_cwds_stat ();
     stat_r = stat (new_name, &s);
@@ -726,10 +771,12 @@ void ren_cmd ()
 	    unmark_file (cpanel);
 	} else {
 	    message (1," Error "," Can't rename multiple files to same name ");
+	    free (new_name);
 	    return;
 	}
     } else if (!S_ISDIR (s.st_mode)){
 	message (1, " Error ", " Destination is not a directory or a file ");
+	free (new_name);
 	return;
     } else {
 	/* destination is a directory */
@@ -746,32 +793,33 @@ void ren_cmd ()
 	} else
 	    move_file_dir (selection->fname, new_name);
     }
+    free (new_name);
     update_panels (UP_OPTIMIZE, 0, 0);
     repaint_screen (RP_NOCLEAR);
 }
 
 void mkdir_cmd ()
 {
-    int current_mask;
-    
     char *dir = input_dialog (" Mkdir ", "Enter directoy name:" , "");
     if (!dir)
 	return;
 
     save_cwds_stat ();
     if (mkdir (dir, 0777) == 0){
-	update_panels (UP_OPTIMIZE, (char *) -1, 0);
+	update_panels (UP_OPTIMIZE, UP_KEEPSEL, 0);
+	unselect_item (cpanel);
+	try_to_select (cpanel, dir);
 	repaint_screen (RP_NOCLEAR);
+	free (dir);
 	return;
     }
+    free (dir);
     message (1, " Error ", "  %s  ", sys_errlist [errno]);
 }
 
 void delete_cmd ()
 {
-    char *dest;
     int  i;
-    struct stat s;
 
     save_cwds_stat ();
     if (cpanel->marked > 0){
@@ -790,7 +838,7 @@ void delete_cmd ()
 	else
 	    erase_file (selection->fname);
     }
-    update_panels (UP_OPTIMIZE, 0, 0);
+    update_panels (UP_OPTIMIZE, UP_KEEPSEL, 0);
     repaint_screen (RP_NOCLEAR);
 }
 
@@ -803,7 +851,7 @@ void ext_cmd ()
 {
     char buffer [200];
     
-    sprintf (buffer, "%s/.nc.ext");
+    sprintf (buffer, "%s/.nc.ext", home_dir);
     do_edit (buffer);
 }
 
@@ -822,6 +870,7 @@ void select_cmd ()
 	c = regexp_match (reg_exp, cpanel->dir [i].fname);
 	if (c == -1){
 	    message (1, " Error ", "  Malformed regular expression  ");
+	    free (reg_exp);
 	    return;
 	}
 	if (c){
@@ -835,6 +884,7 @@ void select_cmd ()
     paint_panel (cpanel);
     panel_refresh (cpanel);
     panel_refresh (cpanel);
+    free (reg_exp);
 }
 
 void unselect_cmd ()
@@ -852,6 +902,7 @@ void unselect_cmd ()
 	c = regexp_match (reg_exp, cpanel->dir [i].fname);
 	if (c == -1){
 	    message (1, " Error ", "  Malformed regular expression  ");
+	    free (reg_exp);
 	    return;
 	}
 	if (c){
@@ -865,6 +916,7 @@ void unselect_cmd ()
     paint_panel (cpanel);
     panel_refresh (cpanel);
     panel_refresh (cpanel);
+    free (reg_exp);
 }
 
 /* Switches to full display panel, well, not so full */
@@ -955,7 +1007,7 @@ void unsorted_cmd ()
 
 void reread_cmd ()
 {
-    update_panels (UP_RELOAD, (char *) -1, 0);
+    update_panels (UP_RELOAD, UP_KEEPSEL, 0);
     repaint_screen (RP_NOCLEAR);
 }
 
@@ -1034,6 +1086,8 @@ menu_entry OptMenu [] = {
     "  Auto save setup",   'A', toggle_auto_save,
     "  auto Menus",        'M', toggle_auto_menu,
     "  use Internal view", 'I', toggle_internal,
+    "  miX all files",     'X', toggle_mix_all_files,
+    "  Fast dir-reload",   'F', toggle_fast_reload,
 /*    "  Align extensions",  'A', toggle_align_extensions, */
 };
 
@@ -1083,7 +1137,9 @@ void menu_cmd ()
     check_options_menu (7, auto_save_setup);
     check_options_menu (8, auto_menu);
     check_options_menu (9, use_internal_view);
-
+    check_options_menu (10, mix_all_files);
+    check_options_menu (11, fast_reload);
+    
     check_menu_panel (&left_panel, PanelMenu);
     check_menu_panel (&right_panel, RightMenu);
     panel = current_panel == &right_panel ? 4 : 0;
@@ -1093,6 +1149,24 @@ void menu_cmd ()
 
 /* Flag toggling functions */
 
+void toggle_fast_reload ()
+{
+    fast_reload = !fast_reload;
+    if (fast_reload_w){
+	message (0, " Information ",
+		 " Using the fast reload option may not reflect the exact "
+		 " directory contents. In this cases you'll need to do a  "
+		 " manual reload of the directory. See the man page for   "
+		 " the details.");
+    }
+}
+
+void toggle_mix_all_files ()
+{
+    mix_all_files = !mix_all_files;
+    update_panels (UP_RELOAD, UP_KEEPSEL, 0);
+}
+
 void toggle_internal ()
 {
     use_internal_view = !use_internal_view;
@@ -1106,13 +1180,13 @@ void toggle_auto_menu ()
 void toggle_show_backup ()
 {
     show_backups = !show_backups;
-    update_panels (UP_RELOAD, (char *) -1, 0);
+    update_panels (UP_RELOAD, UP_KEEPSEL, 0);
 }
 
 void toggle_show_hidden ()
 {
     show_dot_files = !show_dot_files;
-    update_panels (UP_RELOAD, (char *)-1, 0);
+    update_panels (UP_RELOAD, UP_KEEPSEL, 0);
 }
 
 void toggle_verbose ()
@@ -1189,14 +1263,28 @@ void copy_prog_name ()
     handle_char (&cmdline, ' ');
 }
 
+/* Inserts the selected file name in the other panel into the input line */
+void copy_other_prog_name ()
+{
+    char *k = opanel->dir [opanel->selected].fname;
+
+    if ((opanel->view_type==view_info) || (opanel->view_type & VIEW_DISABLED))
+	return;
+	
+    while (*k)
+	handle_char (&cmdline, *k++);
+    
+    handle_char (&cmdline, ' ');
+}
+
 void suspend_cmd ()
 {
     save_cwds_stat ();
     pre_exec ();
     kill (getpid (), SIGTSTP);
     post_exec ();
-    update_panels (UP_OPTIMIZE, (char *) -1, 0);
-    repaint_screen (RP_CLEAR);
+    update_panels (UP_OPTIMIZE, UP_KEEPSEL, 0);
+/*    repaint_screen (RP_CLEAR); */
 }
 
 void quote_next ()
@@ -1234,15 +1322,19 @@ struct {
     KEY_PPAGE,  prev_page,
     XCTRL('l'), repaint_screen_cmd, /* C-l */
 
-    0x80| '\n', copy_prog_name,
+    ALT('\n'),  copy_prog_name,
+    ALT('\t'),  copy_other_prog_name,
     XCTRL('t'), mark_file,
     
     /* Emacs-like bindings */
     XCTRL('v'), next_page,      /* C-v like emacs */
+    ALT('v'),   prev_page,	/* M-v like emacs */
     XCTRL('p'), move_up,	/* C-p like emacs */
     XCTRL('n'), move_down,	/* C-n like emacs */
-    XCTRL('q'), quote_next,
-
+    XCTRL('q'), quote_next,	/* C-q like emacs */
+    XCTRL('s'), start_search,	/* C-s like emacs */
+    ALT('s'),   start_search,	/* M-s not like emacs */
+    
     /* Suspend */
     XCTRL('z'), suspend_cmd,
     
@@ -1268,6 +1360,9 @@ void save_setup ()
     set_int (profile, "auto_menu", auto_menu);
     set_int (profile, "use_internal_view", use_internal_view);
     set_int (profile, "clear_before_exec", clear_before_exec);
+    set_int (profile, "mix_all_files", mix_all_files);
+    set_int (profile, "fast_reload", fast_reload);
+    set_int (profile, "fast_reload_msg_shown", fast_reload_w);
     WritePrivateProfileString ("Dirs", "other_dir", opanel->cwd, profile);
     sync_profiles ();
 }
@@ -1288,7 +1383,7 @@ void load_setup ()
     if (exist_file (buffer)){
 	profile = buffer;
     } else if (exist_file (PROFILE)){
-	profile = profile;
+	profile = PROFILE;
     } else
 	return;
     
@@ -1302,6 +1397,9 @@ void load_setup ()
     auto_save_setup = get_int (profile, "auto_save_setup", 1);
     align_extensions= get_int (profile, "align_extensions", 1);
     auto_menu       = get_int (profile, "auto_menu", 0);
+    fast_reload     = get_int (profile, "fast_reload", 0);
+    fast_reload_w   = get_int (profile, "fast_reload_msg_shown", 0);
+    mix_all_files   = get_int (profile, "mix_all_files", 0);
     use_internal_view=get_int (profile, "use_internal_view", 1);
     clear_before_exec=get_int (profile, "clear_before_exec", 1);
     
@@ -1332,8 +1430,7 @@ void do_nc ()
     init_entry ();
     init_panels ();
     init_menu ();
-    create_input (&cmdline, strlen (prompt), 0, sizeof (input_buffer),
-		  input_buffer, cmdline_win, 0, COLS-2-strlen (prompt), "");
+    create_input (&cmdline, strlen (prompt), 0, cmdline_win, 0, COLS-strlen (prompt), "");
 
     if (auto_menu)
 	user_menu_cmd ();
@@ -1347,8 +1444,8 @@ void do_nc ()
 	}
 	for (i = 0; key_map [i].key_code; i++){
 	    if (key == key_map [i].key_code){
+		searching = 0;
 		(*key_map [i].fn)(key);
-		search_buffer [0] = 0;
 		break;
 	    }
 	}
@@ -1374,8 +1471,7 @@ main (int argc, char *argv [])
     extern int optind, opterr;
     char   *this_dir = 0;
     int    c;
-    
-    initscr ();
+    int    output1, output2;
 
     while ((c = getopt (argc, argv, "Vbs")) != -1){
 	switch (c){
@@ -1387,9 +1483,16 @@ main (int argc, char *argv [])
 	    disable_colors = 1;
 	    break;
 
+case 'P':
+	    print_last_wd = 1;
+	    break;
+	    
 	case 's':
 	    force_slow = 1;	/* Currently it does nothing */
 	    break;
+
+	default:
+	    exit (1);
 	}
     }
 
@@ -1405,6 +1508,21 @@ main (int argc, char *argv [])
     }
     if (this_dir)
 	chdir (this_dir);
+    if (print_last_wd){
+	output1 = dup (1);
+	close (1);
+	output2 = open ("/dev/tty", O_RDWR);
+    }
+    save_stop_handler ();
+    initscr ();
     do_nc ();
     endwin ();
+    if (print_last_wd){
+	close (output2);
+	output2 = dup (output1);
+	close (output1);
+	if (isatty (1))
+	    printf ("%s", cpanel->cwd);
+    }
+    return 0;
 }
