@@ -25,13 +25,16 @@
 #include "util.h"
 #include "color.h"
 
-static char rcsid [] = "$Header: /usr/users/miguel/c/CVS/nc/screen.c,v 1.3 1994/05/14 21:34:59 miguel Exp $";
+static char rcsid [] = "$Header: /usr/users/miguel/c/CVS/nc/screen.c,v 1.6 1994/06/27 23:49:05 miguel Exp $";
 
 int show_mini_info = 1;		/* Show miniinfo */
 
 int extra_info = show_perm;
 
-static void set_colors (Panel *panel)
+/* If true, then use stat() on the cwd to determine directory changes */
+int fast_reload = 0;
+
+void set_colors (Panel *panel)
 {
     wstandend (panel->win_file);
     if (hascolors){
@@ -46,7 +49,7 @@ void panel_refresh (Panel *panel)
     wrefresh (panel->win_file);
 }
 
-int set_attr (Panel *panel, int hilight, int marked)
+void set_attr (Panel *panel, int hilight, int marked)
 {
     int color;
 
@@ -132,7 +135,7 @@ void repaint_file (Panel *panel, int file_index)
 	    break;
 
 	case show_dates:
-	    wprintw (panel->win_file, "No dates yet");
+	    wprintw (panel->win_file, "%12s", file_date_pck (entry.st_ctime));
 	    break;
 	}
     }
@@ -183,7 +186,7 @@ void display_mini_info (Panel *panel)
     }
 }
 
-paint_dir (Panel *panel)
+void paint_dir (Panel *panel)
 {
     int i;
     int hilight;		/* Color used for hilighting */
@@ -230,7 +233,7 @@ static void adjust_top_file (Panel *panel)
 	panel->top_file = panel->selected;
 }
 
-brief_frame (Panel *panel)
+void brief_frame (Panel *panel)
 {
     panel->cols = COLS/2-2;
     panel->win_file = panel->small_frame;
@@ -246,7 +249,7 @@ brief_frame (Panel *panel)
     wprintw (panel->win_file, " %-18s", "     Name");
 }
 
-full_frame (Panel *panel)
+void full_frame (Panel *panel)
 {
     panel->win_file = panel->small_frame;
     panel->cols = COLS/2-2;
@@ -267,7 +270,7 @@ full_frame (Panel *panel)
     wprintw (panel->win_file, "%10s", "Permission");
 }
 
-long_frame (Panel *panel)
+void long_frame (Panel *panel)
 {
     panel->win_file = panel->big_frame;
     panel->cols = COLS-2;
@@ -304,7 +307,7 @@ long_frame (Panel *panel)
     wprintw (panel->win_file, "%-*s", COLS-60, " Name");
 }
 
-paint_panel (Panel *panel)
+void paint_panel (Panel *panel)
 {
     if (panel->view_type & VIEW_DISABLED)
 	return;
@@ -353,11 +356,22 @@ void Xtry_to_select (Panel *panel, char *name)
 	}
 	return;
     }
+
+    /* Try to select a file near the file that is missing */
+    if (panel->selected >= panel->count){
+	panel->selected = panel->count-1;
+	panel->top_file = panel->selected - panel->lines/2;
+	if (panel->top_file < 0)
+	    panel->top_file = 0;
+    } else
+	return;
+#if OLD_CODE
     panel->selected = 0;
     panel->top_file = 0;
+#endif
 }
 
-init_panel (Panel *panel, int x1, int y1, int x2, int y2)
+void init_panel (Panel *panel, int x1, int y1, int x2, int y2)
 {
     getwd (panel->cwd);
     panel->sort_type = (sortfn *) sort_name;
@@ -391,16 +405,21 @@ void panel_reload (Panel *panel)
     
     if (!PANEL_ISVIEW (panel))
 	return;
-    stat (panel->cwd, &current_stat);
-    if (current_stat.st_ctime == panel->dir_stat.st_ctime &&
-	current_stat.st_mtime == panel->dir_stat.st_mtime)
-	return;
+    if (fast_reload){
+	stat (panel->cwd, &current_stat);
+	if (current_stat.st_ctime == panel->dir_stat.st_ctime &&
+	    current_stat.st_mtime == panel->dir_stat.st_mtime)
+	    return;
+    }
     chdir (panel->cwd);
     panel->count = do_reload_dir (&panel->dir, panel->sort_type, panel->count);
     panel->marked = 0;
+    panel->total  = 0;
     for (i = 0; i < panel->count; i++)
-	if (panel->dir [i].f.marked)
+	if (panel->dir [i].f.marked){
 	    panel->marked++;
+	    panel->total += panel->dir [i].buf.st_size;
+	}
     display_mini_info (panel);
 }
 
