@@ -19,19 +19,27 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
+#include <signal.h>		/* my_system */
 #include <sys/param.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <sys/wait.h>		/* my_system */
+#include <errno.h>		/* my_system */
 #include <time.h>
 #include <pwd.h>
 #include <grp.h>
-#ifdef linux
+#include <string.h>
+#if defined(linux) || defined(gnu_regex)
+#ifdef use_local_rx
+#include "rx.h"
+#else
 #include <regex.h>
 #endif
+#endif
 #include "util.h"
 #include "global.h"
 
-static char rcsid [] = "$Header: /usr/users/miguel/c/CVS/nc/util.c,v 1.2 1994/05/14 21:35:05 miguel Exp $";
+static char rcsid [] = "$Header: /usr/users/miguel/c/CVS/nc/util.c,v 1.3 1994/06/27 22:53:29 miguel Exp $";
 
 static char app_text [] = "MouseLess-Commander";
 
@@ -67,8 +75,6 @@ int msglen (char *text, int *lines)
 
 char *trim (char *s, char *d, int len)
 {
-    char *ptr;
-  
     if (strlen (s) > len){
 	strncpy (d, s, len-3);
 	d [len-2] = '.';
@@ -169,10 +175,9 @@ int regexp_match (char *pattern, char *string)
     char *s, *d;
     char new_pattern [100];
     char *x;
-#ifdef linux
+#if defined(linux) || defined (gnu_regex)
     static regex_t r;
-    static char *old;
-    static int first_time = 1;
+    int    rval;
 #endif
 
     x = pattern;
@@ -198,18 +203,12 @@ int regexp_match (char *pattern, char *string)
 	*d = 0;
 	pattern = new_pattern;
     }
-#ifdef linux
-    if (old != x){
-	if (first_time)
-	    first_time = 0;
-	else
-	    regfree (&r);
-        if (regcomp (&r, pattern, REG_EXTENDED|REG_NOSUB)){
-	    return -1;
-        }
-    }
-    old = x;
-    return !regexec (&r, string, 0, NULL, 0);
+#if defined(linux) || defined(gnu_regex)
+    if (regcomp (&r, pattern, REG_EXTENDED|REG_NOSUB))
+	return -1;
+    rval = !regexec (&r, string, 0, NULL, 0);
+    regfree (&r);
+    return rval;
 #else
     if (re_comp (pattern)){
 	return -1;
@@ -265,7 +264,7 @@ void *xmalloc (int size, char *source)
     void *m = malloc (size);
 
     if (!m){
-	fprintf (stderr, "memory exhausted in module (%s)\n");
+	fprintf (stderr, "memory exhausted in module (%s)\n", source);
 	exit (1);
     }
     return m;
@@ -359,6 +358,13 @@ char *file_date (time_t when)
     return &timebuf [4];
 }
 
+/* Like file_date, but packs the data to fit in 10 columns */
+char *file_date_pck (time_t when)
+{
+    /* FIXME: Should return only 10 chars, not 14 */
+    return file_date (when);
+}
+
 char *extract_line (char *s, char *top)
 {
     static char tmp_line [500];
@@ -385,3 +391,52 @@ char *icase_search (char *text, char *data)
     }
     return 0;
 }
+
+static struct sigaction startup_handler;
+
+int my_system (char *shell, char *command)
+{
+    struct sigaction ignore, save_intr, save_quit, save_stop;
+    sigset_t child_mask, save_mask;
+    pid_t pid;
+    int   status;
+
+    ignore.sa_handler = SIG_IGN;
+    sigemptyset (&ignore.sa_mask);
+    ignore.sa_flags = 0;
+    sigaction (SIGINT,  &ignore, &save_intr);
+    sigaction (SIGQUIT, &ignore, &save_quit);
+    sigaction (SIGTSTP, &startup_handler, &save_stop);
+    
+    sigemptyset (&child_mask);
+    sigaddset (&child_mask, SIGCHLD);
+    sigprocmask (SIG_BLOCK, &child_mask, &save_mask);
+    if ((pid = fork ()) < 0){
+	fprintf (stderr, "\n\nfork () = -1\n");
+	return -1;
+    }
+    if (pid == 0){
+	sigaction (SIGINT,  &save_intr, NULL);
+	sigaction (SIGQUIT, &save_quit, NULL);
+	sigprocmask (SIG_SETMASK, &save_mask, NULL);
+
+	execl (shell, shell, "-c", command, (char *) 0);
+	_exit (127);		/* Exec error */
+    } else {
+	while (waitpid (pid, &status, 0) < 0)
+	    if (errno != EINTR){
+		status = -1;
+		break;
+	    }
+    }
+    sigaction (SIGINT,  &save_intr, NULL);
+    sigaction (SIGQUIT, &save_quit, NULL);
+    sigaction (SIGTSTP, &save_stop, NULL);
+    sigprocmask (SIG_SETMASK, &save_mask, NULL);
+    return status;
+}
+
+void save_stop_handler ()
+{
+    sigaction (SIGTSTP, NULL, &startup_handler);
+}
