@@ -24,8 +24,13 @@ char *load_file (char *filename);
 char *get_group (int);
 char *get_owner (int);
 char *file_date (time_t);
+char *file_date_pck (time_t);
+int exist_file (char *name);
 
 /* Returns a copy of *s until a \n is found and is below top */
 char *extract_line (char *s, char *top);
 char *icase_search (char *text, char *data);
 int regexp_match (char *pattern, char *string);
+
+int my_system (char *shell, char *command);
+void save_stop_handler ();
