@@ -18,9 +18,9 @@
 #include "win.h"
 #include "color.h"
 
-static char rcsid [] = "$Header: /usr/users/miguel/c/CVS/nc/win.c,v 1.1.1.1 1994/05/06 19:14:27 miguel Exp $";
+static char rcsid [] = "$Header: /usr/users/miguel/c/CVS/nc/win.c,v 1.2 1994/06/27 22:53:35 miguel Exp $";
 
-wclr (WINDOW *w)
+void wclr (WINDOW *w)
 {
     int x, y;
     
