@@ -1,3 +1,3 @@
-int wclr (WINDOW *w);
+void wclr (WINDOW *w);
 void define_label (WINDOW *fkeys, int index, char *text);
 WINDOW *new_fkey ();
